# Generate Command Library
We recommend using the latest command dictionary as a starting point. This will allow you to use the most up-to-date commands that have been tested and validated. When you upload a command dictionary, an autogenerated `command_library` typescript file will be available to download. This library will be used later.

## Upload
Uploading a command dictionary is very simple using graphQL. In order to upload a command dictionary, login to the Aerie UI and navigate to the GraphQL Playground. From there, you will use our pre-defined mutation to upload the command dictionary. Run this graphQL mutation below to upload the command dictionary.

```
mutation Upload(
  $commandDictionaryString: String!
) {
  uploadDictionary(dictionary: $commandDictionaryString) {
    id
    command_types_typescript_path
    mission
    version
    created_at
    updated_at
  }
}
```

### Return Values

* id - unique id of the database entry
* command_types_typescript_path - path where the generated command library is stored for the upload dictionary
* mission - the mission name for the command dictionary uploaded
* version - The version of the command dictionary uploaded
* created_at - when was the command dictionary first uploaded
* updated_at - when was the command dictionary updated thus generating a new command library

## Download Command Library

A successful upload will return an `id` used to retrieve the library. Running the graphQL query below will retrieve the typescript library.

1. Run this graphQL command below

```
query GetCommandTypescript($commandDictionaryId: String!) {
  getCommandTypeScript(commandDictionaryId: commandDictionaryId) {
    reason
    status
    typescriptFiles {
      content
      filePath
    }
  }
}
```

2. Save this to a file named `command-types.ts`

### Return Values

* content - The typescript library for your activity_type
* filepath - the filename of your library
* reason - If any errors were thrown when generating this library file, the error message would be stored here. If there are no errors, you will get a `null`
* status - server-status ex. 404, 500, 200
